@@ -285,10 +285,7 @@ impl RaknetSocket {
         let peer_addr = self.peer_addr;
         let local_addr = self.local_addr;
         let sendq = self.sendq.clone();
-        let socket = self.s.clone();
         let recvq = self.recvq.clone();
-        let enable_loss = self.enable_loss.clone();
-        let loss_rate = self.loss_rate.clone();
         let last_heartbeat_time = self.last_heartbeat_time.clone();
         let incomming_notify = self.incomming_notify.clone();
         tokio::spawn(async move {
@@ -316,13 +313,15 @@ impl RaknetSocket {
                     //handle ack
                     let mut sendq = sendq.lock().await;
                     let ack = read_packet_ack(&buf).await.unwrap();
-                    if ack.single_sequence_number{
-                        sendq.ack(ack.sequences.0);
-                    } else{
-                        for i in ack.sequences.0..ack.sequences.1+1{
-                            sendq.ack(i);
+                    for i in 0..ack.record_count{
+                        if ack.sequences[i as usize].0 == ack.sequences[i as usize].1{
+                            sendq.ack(ack.sequences[i as usize].0);
+                        } else{
+                            for i in ack.sequences[i as usize].0..ack.sequences[i as usize].1+1{
+                                sendq.ack(i);
+                            }
+                            
                         }
-                        
                     }
                     continue;
                 }
@@ -331,12 +330,15 @@ impl RaknetSocket {
                     //handle nack
                     let nack  = read_packet_nack(&buf).await.unwrap();
 
-                    if nack.single_sequence_number {
-                        sendq.lock().await.nack(nack.sequences.0 , cur_timestamp_millis());
-                    } else {
-                        let mut sendq = sendq.lock().await;
-                        for i in nack.sequences.0..nack.sequences.1+1{
-                            sendq.nack(i, cur_timestamp_millis());
+                    let mut sendq = sendq.lock().await;
+
+                    for i in 0..nack.record_count{
+                        if nack.sequences[i as usize].0 == nack.sequences[i as usize].1 {
+                            sendq.nack(nack.sequences[i as usize].0 , cur_timestamp_millis());
+                        } else {
+                            for i in nack.sequences[i as usize].0..nack.sequences[i as usize].1+1{
+                                sendq.nack(i, cur_timestamp_millis());
+                            }
                         }
                     }
                     continue;
@@ -366,22 +368,6 @@ impl RaknetSocket {
                         }
                     }
 
-                    for nack in recvq.get_nack(){
-                        let nack = NACK{
-                            // always 1
-                            record_count: 1,
-                            single_sequence_number: nack.0 == nack.1,
-                            sequences: nack,
-                        };
-
-                        let buf = write_packet_nack(&nack).await.unwrap();
-                        match RaknetSocket::sendto(&socket , &buf, &peer_addr , &enable_loss , &loss_rate).await{
-                            Ok(_) => {},
-                            Err(_) => {
-                                break;
-                            },
-                        };
-                    }
                 } else {
                     raknet_log!("unknown packetid : {}", buf[0]);
                 }
@@ -409,20 +395,33 @@ impl RaknetSocket {
                 let mut recvq = recvq.lock().await;
                 let acks = recvq.get_ack();
 
-                for ack in acks {
-
-                    let single_sequence_number = ack.1 == ack.0;
+                if acks.len() != 0 {
 
                     let packet = ACK{
-                        // catch minecraft 1.18.2 client packet , the parameter always 1
-                        record_count: 1,
-                        single_sequence_number,
-                        sequences: ack,
+                        record_count: acks.len() as u16,
+                        sequences: acks,
                     };
 
                     let buf = write_packet_ack(&packet).await.unwrap();
                     RaknetSocket::sendto(&s , &buf, &peer_addr , &enable_loss  , &loss_rate).await.unwrap();
                 }
+
+                // flush nack
+                let nacks = recvq.get_nack();
+                if nacks.len() != 0{
+                    let nack = NACK{
+                        record_count: nacks.len() as u16,
+                        sequences: nacks,
+                    };
+
+                    let buf = write_packet_nack(&nack).await.unwrap();
+                    match RaknetSocket::sendto(&s , &buf, &peer_addr , &enable_loss , &loss_rate).await{
+                        Ok(_) => {},
+                        Err(_) => {
+                            break;
+                        },
+                    };
+                }
                 
                 //flush sendq
                 let mut sendq = sendq.lock().await;
@@ -603,11 +602,11 @@ impl RaknetSocket {
 
     /// Set the packet loss rate and use it for testing
     /// 
-    /// The `stage` parameter ranges from 1 to 10, indicating a packet loss rate of 10% to 20%.
+    /// The `stage` parameter ranges from 1 to 10, indicating a packet loss rate of 90% to 0%.
     /// # Example
     /// ```ignore
     /// let mut socket = RaknetSocket::connect("127.0.0.1:19132".parse().unwrap()).await.unwrap();
-    /// // set 80% loss packet rate.
+    /// // set 20% loss packet rate.
     /// socket.set_loss_rate(8);
     /// ```
     pub fn set_loss_rate(&mut self ,stage : u8){
